/*
// File:		MainForm.cs
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2008 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Emai;
using Emai.Pop3;
using Emai.Mime;

namespace Pop3
{
	public partial class MainForm : Form
	{
		private System.Array messages;
		private UInt32 messageCount;
		private UInt32 currentMessageNumber;

		public MainForm()
		{
			InitializeComponent();
			this.comboBoxAuthType.SelectedIndex = 0;
			this.comboBoxRemoveTime.SelectedIndex = 1;
		}

		private void buttonClose_Click(object sender, EventArgs e)
		{
			Application.Exit();
		}

		private void buttonReceive_Click(object sender, EventArgs e)
		{
			this.listViewMessages.Items.Clear();
			this.listViewAttachments.Items.Clear();
			this.textBoxPlainText.Text = "";
			this.textBoxHtmlText.Text = "";
			this.textBoxFrom.Text = "";
			this.textBoxTo.Text = "";
			this.textBoxCC.Text = "";
			this.messageCount = 0;
			this.currentMessageNumber = 0;
			try
			{
				Emai.License.Key = this.textBoxLicKey.Text;
				UInt32 options = (UInt32)EnumNullOptions.NullOptions;
				if (this.checkBoxUseSSL.Checked)
					options = (UInt32)EnumPop3SSL.Pop3UseSSLOption;
				Pop3Session session = new Pop3Session(60, options);
				session.OnOperation += new Event_OnOperationEventHandler(this.mailit_OnOperation);
				session.OnDataReceived += new Event_OnDataEventHandler(this.mailit_OnDataReceived);
				session.OnServerError += new Event_OnServerErrorEventHandler(this.mailit_OnServerError);
				session.Connect(this.textBoxMailServer.Text);

				UInt32[] authTypes = {
			    (UInt32)EnumPop3AuthSelector.POP3AuthAutomatic,
			    (UInt32)EnumPop3AuthSelector.POP3AuthLoginCommand,
			    (UInt32)EnumPop3AuthSelector.POP3AuthApop,
			    (UInt32)EnumPop3AuthSelector.POP3AuthLogin,
			    (UInt32)EnumPop3AuthSelector.POP3AuthPlain,
			    (UInt32)EnumPop3AuthSelector.POP3AuthCramMD5,
			    (UInt32)EnumPop3AuthSelector.POP3AuthNTLM };

				session.Authenticate(this.textBoxAccount.Text, this.textBoxPassword.Text, authTypes[this.comboBoxAuthType.SelectedIndex]);

				MessageInfoList messageInfoList = new MessageInfoList(Application.StartupPath + "\\net.dacons.pop3demo.messagehistory", (UInt32)EnumNullOptions.NullOptions);
				session.RetrieveMessageList(messageInfoList, (UInt32)EnumNullOptions.NullOptions);

				this.messageCount = messageInfoList.GetCount((UInt32)EnumPop3MessageInfoListGetCountFlags.MessagesForDownload);
				this.messages = Array.CreateInstance(typeof(Emai.Mime.Message), this.messageCount);
				while (this.currentMessageNumber < this.messageCount)
				{
					MessageInfo outInfo = new MessageInfo();
					messageInfoList.GetInfo(this.currentMessageNumber, ref outInfo, (UInt32)EnumPop3MessageInfoListGetCountFlags.MessagesForDownload);

					Emai.Mime.Message message = session.RetrieveMessage(outInfo.Number, (UInt32)EnumNullOptions.NullOptions);
					messages.SetValue(message, this.currentMessageNumber);
					messageInfoList.MarkMessageReceived(++this.currentMessageNumber, (UInt32)EnumNullOptions.NullOptions);
				}

				// Remove message from server if chose
				if (this.checkBoxRemoveTime.Checked)
				{
					UInt32[] numDays = { 1, 7, 31 };

					messageInfoList.EvaluateSublist(numDays[this.comboBoxAuthType.SelectedIndex], (UInt32)EnumPop3MessageInfoListGetCountFlags.MessagesForDeletion);

					this.currentMessageNumber = 0;
					this.messageCount = messageInfoList.GetCount((UInt32)EnumPop3MessageInfoListGetCountFlags.MessagesForDeletion);

					while (this.currentMessageNumber < this.messageCount)
					{
						MessageInfo outInfo = new MessageInfo();
						messageInfoList.GetInfo(this.currentMessageNumber, ref outInfo, (UInt32)EnumPop3MessageInfoListGetCountFlags.MessagesForDeletion);

						// Delete message with number outInfo.messageNumber
						session.DeleteMessage(outInfo.Number);
						this.currentMessageNumber++;
					}
				}

				session.Disconnect((UInt32)EnumNullOptions.NullOptions);
			}
			catch (InvalidLicenseException exception)
			{
				MessageBox.Show("Licence key is invalid. Restart application and enter valid key.", "Error");
				this.toolStripStatusLabelText.Text = "";
				Console.Out.WriteLine(exception.ErrorCode);
			}
			catch (Emai.Exception exception)
			{
				this.toolStripStatusLabelText.Text = "";
				Console.Out.WriteLine(exception.ErrorCode);
			}

			if (messages.Length > 0)
			{
				String subject;
				for (UInt32 i = 0; i <= messages.Length - 1; i++)
				{
					try
					{
						Emai.Mime.Message message = (Emai.Mime.Message)messages.GetValue(i);
						subject = message.GetField(null, (UInt32)EnumMessageFieldType.Subject);
					}
					catch (Emai.Exception /*exception*/)
					{
						subject = "Error while getting subject";
					}
					this.listViewMessages.Items.Add(subject);
				}
				this.listViewMessages.Items[this.listViewMessages.Items.Count - 1].Selected = true;
				this.listViewMessages.TopItem = this.listViewMessages.Items[this.listViewMessages.Items.Count - 1];
			}
			this.toolStripStatusLabelText.Text = "";
			this.toolStripProgressBarPercent.Value = 0;
		}

		private void buttonSave_Click(object sender, EventArgs e)
		{
			if (this.listViewAttachments.Items.Count > 0)
			{
				ListView.SelectedListViewItemCollection msgs = this.listViewMessages.SelectedItems;
				Emai.Mime.Message message;
				foreach (ListViewItem item in msgs)
				{
					message = (Emai.Mime.Message)messages.GetValue(item.Index);
					Common.MessagePartEnumerator enumerator = new Common.MessagePartEnumerator(message);
					enumerator.Enumerate();
					ListView.SelectedListViewItemCollection attachments = this.listViewAttachments.SelectedItems;
					foreach (ListViewItem i in attachments)
					{
						String fileName = "";
						String contentID = "";
						Byte[] data = new Byte[1];
						enumerator.GetAttachmentAtIndex(i.Index, ref data, ref fileName, ref contentID);
						this.saveFileDialog.FileName = fileName;
						if (this.saveFileDialog.ShowDialog() == DialogResult.OK)
						{
							FileStream file = (FileStream)this.saveFileDialog.OpenFile();
							file.Write(data, 0, data.Length);
							file.Close();
						}
					}
				}
			}
		}

		private void checkBoxRemoveTime_CheckedChanged(object sender, EventArgs e)
		{
			this.comboBoxRemoveTime.Enabled = this.checkBoxRemoveTime.Checked;
		}

		private void messagesSelectedIndexChanged(object sender, EventArgs e)
		{
			ListView.SelectedListViewItemCollection msgs = this.listViewMessages.SelectedItems;
			this.listViewAttachments.Items.Clear();
			Emai.Mime.Message message;
			foreach (ListViewItem item in msgs)
			{
				message = (Emai.Mime.Message)messages.GetValue(item.Index);
				Common.MessagePartEnumerator enumerator = new Common.MessagePartEnumerator(message);
				enumerator.Enumerate();
				this.textBoxFrom.Text = enumerator.From;
				this.textBoxTo.Text = enumerator.To;
				this.textBoxCC.Text = enumerator.CC;
				this.textBoxPlainText.Text = enumerator.Text;
				this.textBoxHtmlText.Text = enumerator.Html;
				for (Int32 i = 0; i < enumerator.AttachmentsCount; i++)
				{
					String fileName = "";
					String res = "";
					String contentID = "";
					Byte[] data = new Byte[1];
					enumerator.GetAttachmentAtIndex(i, ref data, ref fileName, ref contentID);
					if (contentID.Length > 0)
						res = String.Format("{0:S} (size = {1:D} bytes content ID = {2:S})", fileName, data.Length, contentID);
					else
						res = String.Format("{0:S} (size = {1:D} bytes)", fileName, data.Length);
					this.listViewAttachments.Items.Add(res);
				}
			}
		}

		private void mailit_OnDataReceived(object sender, DataEventArgs e)
		{
			this.toolStripProgressBarPercent.Value = 0;
			Int32 curPercent = (100 * (Int32)e.TransferedSize) / (Int32)e.TotalSize;
			this.toolStripProgressBarPercent.Value = curPercent;
		}

		private void mailit_OnServerError(object sender, ServerErrorEventArgs e)
		{
			this.toolStripStatusLabelText.Text = "The server error encountered was: " + e.ServerReply;
		}

		private void mailit_OnOperation(object sender, OperationEventArgs e)
		{
			String[] statusTexts = {
			"Connecting to server...",
			"Successfully connected to server.",
			"Authenticating on server...",
			"Successfully authenticated on server.",
			"Getting message list from server...",
			"Successfully got messages from server.",
			"Getting message UIDLs...",
			"Successfully got messages UIDLs.",
			"Retrieving message {0:D} of {1:D}.",
			"Successfully retrieved messages.",
			"Deletting message {0:D} of {1:D}.",
			"Message successfully deleted.",
			"Sending NOOP...",
			"NOOP successfully sent.",
			"Closing connection.",
			"Connection successfully closed."
	        };
			switch (e.OperationCode)
			{
				case (UInt32)EnumPop3OperationCode.Pop3RetrievingMessage:
				case (UInt32)EnumPop3OperationCode.Pop3DeletingMessage:
					this.toolStripStatusLabelText.Text = String.Format(statusTexts[e.OperationCode],
						this.currentMessageNumber + 1, this.messageCount);
					break;
				default:
					this.toolStripStatusLabelText.Text = statusTexts[e.OperationCode];
					break;
			}
			this.statusStrip.Refresh();
		}

		private void messagesListResized(object sender, EventArgs e)
		{
			this.columnHeaderSubject.Width = this.listViewMessages.ClientSize.Width;
		}

		private void textBoxLicKey_Enter(object sender, EventArgs e)
		{
			if (this.textBoxLicKey.Text.CompareTo("Put your license key here.") == 0)
			{
				this.textBoxLicKey.Text = "";
			}
		}

		private void textBoxLicKey_Leave(object sender, EventArgs e)
		{
			if (this.textBoxLicKey.Text.Length == 0)
			{
				this.textBoxLicKey.Text = "Put your license key here.";
			}
		}
	}
}